open Ast 
open Graphics



module NameMap = Map.Make(struct
   type t = string
   let compare x y = Pervasives.compare x y   end)

exception ReturnException of int * int NameMap.t


       
(*   begin of run   *)
let run (vars, funcs)=
  
  
               (* Put function declarations in a symbol table *)
  let func_map = List.fold_left (fun fmap fun_declist ->NameMap.add fun_declist.fname fun_declist fmap) NameMap.empty funcs
  in 
   
  
                                       (*call function*)
 
  let rec call fun_declist globals=    
    

                                    (*function for evaluating arithmatic expression, returning the value and environment*)
    let rec eval_arith env=function
        Literal(i) -> i, env 
      | Id(var)-> let globals= env 
                  in
                  if NameMap.mem var globals then 
                                begin
                                if ((NameMap.find var globals).vtype="int") then 
                                       (NameMap.find var globals).ivalue, env
                                else
                                       raise (Failure ("this identifier is not of type int"))
                                end
                  else raise (Failure ("undeclared identifier " ^ var))
 
      | Arith(e1, op, e2) -> let v1, env = eval_arith  env e1 
                               in
                               let v2, env = eval_arith env e2 
                               in
                              
                              (match op with
                                  Add -> v1 + v2
                                | Sub -> v1 - v2
                                | Mult-> v1 * v2
                                | Div ->  v1 / v2), env

    in
                         (*
function for evaluating equality test part*)
    let rec eval_equality env=function
        
Eqtest(ae1,ae2)->let v1, env = eval_arith env ae1 
                         in
                         let v2, env = eval_arith env ae2
                         in
                         let boolean i = if i then 1 else 0 
                         in boolean (v1 = v2), env

   
    in 
                        (*function for executing expressions*)
    let rec exec_exp env=function
        Assign_i(var,ae)-> let v, globals = eval_arith env ae 
                        in
                        let irecord ={ vname= var; vtype="int" ; ivalue=v ; svalue="none"}
                        in
                           if NameMap.mem var globals then 
                                begin
                                if (NameMap.find var globals).vtype="int" then 
                                     (NameMap.add var irecord globals) 
                                else 
                                      raise(Failure("identifier is not of type int"))
                                end
                           else raise (Failure ("undeclared identifier" ^ var))


      |Assign_s(var, s)->let globals =env
                       in  
                       let srecord={ vname= var; vtype="string" ; ivalue=0 ; svalue=s}
                       in
                          if NameMap.mem var globals then 
                                begin
                                if (NameMap.find var globals).vtype="string"then 
                                     (NameMap.add var srecord globals) 
                                else 
                                      raise(Failure("identifier is not of type string"))
                                end
                           else raise (Failure ("undeclared identifier" ^ var))


     |Rec(ae1,ae2,ae3,ae4)->let v1, env = eval_arith env ae1 
                          in
                          let v2, env = eval_arith env ae2 
                          in
                          let v3, env = eval_arith env ae3 
                          in
                          let v4, env = eval_arith env ae4 
                          in
                          Graphics.draw_rect v1 v2 v3 v4;
                          env


     |Elp(ae1,ae2,ae3,ae4)->let v1, env = eval_arith env ae1 
                          in
                          let v2, env = eval_arith env ae2 
                          in
                          let v3, env = eval_arith env ae3 
                          in
                          let v4, env = eval_arith env ae4 
                          in
                          Graphics.draw_ellipse v1 v2 v3 v4;
                          env

     |Line(ae1,ae2,ae3,ae4)->let v1, env = eval_arith env ae1 
                          in
                          let v2, env = eval_arith env ae2 
                          in
                          let v3, env = eval_arith env ae3 
                          in
                          let v4, env = eval_arith env ae4 
                          in
                          Graphics.moveto v1 v2;
                          Graphics.lineto v3 v4;
                         env

     |Txt_s(str,ae1,ae2)->let v1, env = eval_arith env ae1 
                          in
                          let v2, env = eval_arith env ae2 
                          in
                          Graphics.moveto v1 v2;
                          Graphics.draw_string str;
                          env
    
     |Txt_id(str,ae1,ae2)->let v1, env = eval_arith env ae1  (*test weather the id is string or not*)
                          in
                          let v2, env = eval_arith env ae2 
                          in let st=
                             let globals = env 
                             in
                             if NameMap.mem str globals then  
                             begin
                               if (NameMap.find str globals).vtype="string" then
                                    (NameMap.find str globals).svalue 
                               else raise (Failure ("identifier is not a string"))
                             end
                             else raise (Failure ("undeclared identifier" ^ str))
                          in
                          Graphics.moveto v1 v2;
                          Graphics.draw_string st;
                          env 
    
  in
                   (*function for executing stmt part*)
   let rec exec_stmt env = function
       Exp(e)->let env=exec_exp env e
            in env 
       |If(et, slist1, slist2)->let test, env= eval_equality env et
                             in List.fold_left exec_stmt env (if test !=0 then slist1 else slist2) 
                        
       |Loop(i, slist)-> let rec looping env i= 
                           let j= i-1
                            in  
                            if j!=0 then
                             begin  
                             looping  ( List.fold_left exec_stmt env slist ) j
                             end
                            else  env
                         in looping env (i+1)

       |Call_f(str)->if NameMap.mem str func_map
                     then let frecord=NameMap.find str func_map
                          in
                          List.fold_left exec_stmt env frecord.fbody
                     else raise (Failure ("undefine function" ^ str))




in
(*-------------------------------------body of call-------------------------------------------*)

List.fold_left exec_stmt globals  fun_declist.fbody; (*end of call function*)

in

(*-------------------------------------body of run-----------------------------------*)

                                     (* Put variables declarations in a symbol table *) 
let var_map = List.fold_left (fun vmap id_dec ->NameMap.add id_dec.vname id_dec vmap) NameMap.empty vars
in



try
   call(NameMap.find "Object" func_map) var_map
with Not_found ->
raise (Failure ("did not find the Object section function"))




