                                                           
%{ open Ast
   open Printf %}

%token LPAREN  RPAREN  LBRACK  RBRACK SEMI  COMMA 
%token PLUS  MINUS  TIMES  DIVIDE  ASSIGN  EQ 
%token IF  ELSE  ENDIF  LOOP  ENDLOOP  FUNC  ENDFUNC CALLF
%token RED  BLUE  WHITE  BLACK GREEN
%token RECTANGLE  ELLIPSE  LINE  TEXT 
%token STRING  INT  DECLARE  WINDOW  OBJECT  EOF

%token <int> LITERAL
%token <string> ID
%token <string> ST

%nonassoc ELSE
%left  ASSIGN
%left  EQ 
%left PLUS  MINUS
%left TIMES  DIVIDE

%start doodle_program      /* entry point */ 

%type <Ast.doodle_program> doodle_program 

%%


doodle_program:                   /*3 main sections of the program*/
decl_sec window_sec object_sec {( fst $1), ($3 :: snd $1) }
;


decl_sec:                        /*this section is optional */
/*nothing*/ {[],[]}
|DECLARE LBRACK declist RBRACK {$3}
;


declist:                 /*list of declarations*/
/*nothing*/ {[],[]}
|declist id_dec  {($2::fst $1), (snd $1) }    /*add an integer,string variable declaration to the first list*/
|declist fun_dec { fst $1, ($2 :: snd $1) }   /*add function declaration to the second list*/
;


id_dec:                /*declaring an identifier*/
INT ID ASSIGN LITERAL  SEMI   {{ vname= $2; vtype="int" ; ivalue=$4; svalue="none"}}    /*integer variable*/
|STRING ID  ASSIGN ST  SEMI   {{ vname= $2; vtype="string" ; ivalue=0; svalue=$4}} /*string variable*/
; 


fun_dec:
FUNC  ID   stmt_list ENDFUNC 
{{ fname= $2; 
   fbody= List.rev $3}}/*function declaration*/
;

window_sec:   /*winodw section*/
WINDOW  LBRACK   LPAREN LITERAL COMMA LITERAL RPAREN SEMI  color RBRACK  
{Graphics.open_graph " string_of_int($4) ^ 'x' ^ string_of_int($6) ";Graphics.set_color $9}   
;





color:
RED  {0xff0000}   
|BLUE {0x0000ff}   
|WHITE {0xffffff}   
|BLACK {0x000000}   
|GREEN {0x00ff00}   
; 



object_sec:   /*object section */
OBJECT LBRACK stmt_list  RBRACK
{{ fname="Object"; 
   fbody= List.rev $3}}
;

stmt_list:    
/*nothing*/ {[]}
|stmt_list stmt {$2::$1}  
;


stmt:
exp SEMI {Exp($1)} 
|IF  LPAREN eqtest RPAREN stmt_list ELSE  stmt_list ENDIF {If($3, $5, $7)}
|IF  LPAREN eqtest RPAREN stmt_list ENDIF {If($3, $5,[ ] )}
|LOOP LPAREN LITERAL RPAREN stmt_list ENDLOOP{Loop($3,$5) }
|CALLF ID SEMI {Call_f($2)}
;

eqtest:
arith_exp EQ arith_exp {Eqtest($1, $3)}
;

exp:
ID ASSIGN arith_exp{Assign_i($1, $3)}
|ID ASSIGN  ST  {Assign_s($1, $3)}
|RECTANGLE  LPAREN arith_exp  COMMA arith_exp COMMA arith_exp  COMMA arith_exp  RPAREN{Rec($3,$5,$7,$9)}
|ELLIPSE  LPAREN arith_exp  COMMA arith_exp COMMA arith_exp  COMMA arith_exp  RPAREN{Elp($3,$5,$7,$9)}
|LINE     LPAREN arith_exp  COMMA arith_exp COMMA arith_exp  COMMA arith_exp  RPAREN{Line($3,$5,$7,$9)}
|TEXT  LPAREN ST  COMMA arith_exp COMMA arith_exp  RPAREN{Txt_s ($3,$5,$7)}
|TEXT  LPAREN ID COMMA arith_exp COMMA arith_exp  RPAREN{Txt_id ($3,$5,$7)}
;



arith_exp:  /*arithmatic operations, done on integers and id of integers*/
ID {Id($1)}
|LITERAL {Literal($1)} 
|LPAREN arith_exp RPAREN  {$2}
|arith_exp PLUS arith_exp {Arith($1,Add, $3)}
|arith_exp MINUS arith_exp {Arith($1,Sub, $3)}
|arith_exp TIMES  arith_exp {Arith($1,Mult, $3)}
|arith_exp DIVIDE arith_exp {Arith($1,Div, $3)}


